Attribute VB_Name = "Common"
'---------------------------------------------------------------------
'   ==========
'   COMMON.BAS
'   ==========
'
' Common constants and functions shared by Telescope and Focuser
'
' Implements shared connections for the Meade Telescope/Focuser driver.
' This is necessary, because the focuser and telescope commands must
' be issued on the same COM port.
'
' Written:  February 2nd, 2002   Chris Creery <ccreery@cyanogen.com>
'
' Edits:
'
' When          Who What
' ----------    --- --------------------------------------------------
' 02-Feb-2002   CC  1.5.0   Initial edit
' 07-May-2002   rbd 1.8.3   Name changes to g_xxx
' 18-May-2002   rbd 1.8.4   Change to out-of-proc server to support
'                           ownership of Telescope and Focuser by
'                           different processes. Add logic to prevent
'                           creating multiple instances of interfaces.
' 18-May-2002   rbd 1.8.4   Move ID's and DESC here, they are common.
'                           Rename module to Common. Add Who parameter
'                           to OpenConnection() and CloseConnection()
'                           to avoid the need for setting global after
'                           calling OpenConnection() and before
'                           calling CloseConnection(). Move common GS
'                           scope test to OpenConnection(). Move
'                           common port setting from registry here.
' 18-Jul-02     rbd 1.8.6   Two drivers share same soruces, one the
'                           is the DLL version, the other is the EXE.
' 30-Sep-2002   rbd 2.1.0   Fix error handling in OpenConnection()
' 04-Jan-2003   rbd 2.1.2   Reduce initial timeout from 5 to 2 sec,
'                           set 2 sec if exit from OpenConnection()
'                           for "already open".
' 03-Jan-2004   rbd 4.1.0   Move scope detection to Telescope, new
'                           LX200GPS startup will reboot a parked
'                           scope.
' 20-July-2008  GC 4.1.0G   Added second serial port for direct interface
'                           to enhanced 909 and the AS 497
'---------------------------------------------------------------------
Option Explicit

#If EXE_DRIVER = 1 Then
    Public Const DESC As String = "Meade Scope/Focuser w/909 serial"
    Public Const SCOPE_ID As String = "MeadeExG.Telescope"
    Public Const FOCUSER_ID As String = "MeadeExG.Focuser"
#Else
    Public Const DESC As String = "Meade Autostar w/909 serial"
    Public Const SCOPE_ID As String = "MeadeG.Telescope"
    Public Const FOCUSER_ID As String = "MeadeG.Focuser"
#End If

Public Enum DeviceClass
    devTelescope = 0
    devFocuser = 1
End Enum

Public g_SharedSerial As DriverHelper.Serial    ' Shared serial port
'GC
Public g_GuideSerial As DriverHelper.Serial     ' Serial port to 909
Public g_bTelescopeConnected As Boolean         ' Device state flags
Public g_bFocuserConnected As Boolean
Public g_bTelescopeCreated As Boolean           ' Multiple instance interlocks
Public g_bFocuserCreated As Boolean
Public g_Scope As Telescope
'
' Private objects for code in this module
'
Private oProfile As DriverHelper.Profile
Private oUtil As DriverHelper.Util
'
' ==============
' SERVER STARTUP
' ==============
'
Public Sub Main()
    '
    ' Catch the curious who might try to double-click
    ' this executable.
    '
    If App.StartMode = vbSModeStandalone Then
        MsgBox "This is an ASCOM driver. It cannot be run stand-alone", _
                (vbOKOnly + vbCritical + vbMsgBoxSetForeground), App.FileDescription
        Exit Sub
    End If
    '
    ' We are serving objects, initialize all shared resources
    '
    g_bTelescopeConnected = False
    g_bFocuserConnected = False
    g_bTelescopeCreated = False
    g_bFocuserCreated = False
    Set g_SharedSerial = New DriverHelper.Serial
    Set g_GuideSerial = New DriverHelper.Serial
    Set oProfile = New DriverHelper.Profile
    oProfile.DeviceType = "Telescope"
    Set oUtil = New DriverHelper.Util
    
End Sub

Public Sub OpenConnection(Who As DeviceClass)
    Dim buf As String, code As Long, src As String
    
    Select Case Who
        Case devTelescope:
            g_bTelescopeConnected = True
        Case devFocuser:
            g_bFocuserConnected = True
    End Select
'GC
    If g_GuideSerial.Connected Then                        ' Open port only once
        g_GuideSerial.ReceiveTimeout = 1                   ' Caller expects 2 second timeout
    End If
'/GC
    If g_SharedSerial.Connected Then                        ' Open port only once
        g_SharedSerial.ReceiveTimeout = 2                   ' Caller expects 2 second timeout
        Exit Sub
    End If

    buf = oProfile.GetValue(SCOPE_ID, "COM Port")
    If buf = "" Then                                        ' Default to COM1
        buf = "1"
        oProfile.WriteValue SCOPE_ID, "COM Port", buf
    End If
    On Error GoTo SERIAL_PORT_FAILED
    g_SharedSerial.Port = CLng(buf)                         ' Set port
    g_SharedSerial.Speed = ps9600                           ' LX200 uses 9600
    g_SharedSerial.ReceiveTimeout = 2                       ' 2 second timeout
    g_SharedSerial.Connected = True                         ' Grab the serial port

    g_SharedSerial.Transmit "#"                             ' Kick the LX200 GPS in the butt (ESSENTIAL!)
    oUtil.WaitForMilliseconds 500
    g_SharedSerial.ClearBuffers                             ' Clear remaining junk in buffer

'GC
    buf = oProfile.GetValue(SCOPE_ID, "COM PortG")
    If buf = "" Then                                        ' Default to COM1
        buf = "2"
        oProfile.WriteValue SCOPE_ID, "COM PortG", buf
    End If

    On Error GoTo SERIAL_PORT_FAILED1
    g_GuideSerial.Port = CLng(buf)                         ' Set port
    g_GuideSerial.Speed = ps9600                           ' LX200 uses 9600
    g_GuideSerial.ReceiveTimeout = 1                       ' 2 second timeout
    g_GuideSerial.Connected = True                         ' Grab the serial port

    g_GuideSerial.ClearBuffers                             ' Clear remaining junk in buffer
    g_GuideSerial.Transmit "#"                             ' Kick the LX200 GPS in the butt (ESSENTIAL!)
    g_GuideSerial.Transmit Chr$(6)
    On Error GoTo SERIAL_PORT_FAILED1                  ' -- error trap ---
    buf = g_GuideSerial.ReceiveCounted(1)                ' Receive one character
    On Error GoTo 0                                 ' -- end trap ---
    If buf = "A" Then
        Exit Sub
    End If
    g_GuideSerial.Connected = False
    buf = oProfile.GetValue(SCOPE_ID, "COM PortG")
    MsgBox ("909 serial not found on Com Port = " & buf)
    Exit Sub
SERIAL_PORT_FAILED1:
    g_GuideSerial.Connected = False
    buf = oProfile.GetValue(SCOPE_ID, "COM PortG")
    MsgBox ("Could not open 909 serial on Com Port = " & buf)
    Exit Sub
'/GC                        
SERIAL_PORT_FAILED:
    g_bTelescopeConnected = False
    g_bFocuserConnected = False
    buf = Err.Description
    code = Err.Number
    src = Err.Source
    Resume SER_FAIL_FIN
    
SER_FAIL_FIN:
    On Error GoTo 0                                         ' Resignal to callers & client
    Err.Raise code, src, buf

End Sub

Public Sub CloseConnection(Who As DeviceClass)
    
    Select Case Who
        Case devTelescope:
            g_bTelescopeConnected = False
            Set g_Scope = Nothing
        Case devFocuser:
            g_bFocuserConnected = False
    End Select
    
    If Not (g_SharedSerial Is Nothing) Then                 ' If we ever connected
        g_SharedSerial.ClearBuffers                         ' Always clear serial buffers
'GC
        g_GuideSerial.ClearBuffers                         ' Always clear serial buffers
'/GC
        If (Not g_bTelescopeConnected) And (Not g_bFocuserConnected) Then
            g_SharedSerial.Connected = False                ' Release if no longer used
'GC
            g_GuideSerial.Connected = False                ' Release if no longer used
'/GC
        End If
    End If
    
End Sub

Public Function IsConnected(Who As DeviceClass)
    Select Case Who
        Case devTelescope: IsConnected = g_bTelescopeConnected
        Case devFocuser: IsConnected = g_bFocuserConnected
    End Select

End Function
