VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Telescope"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "ASCOM Telescope driver for Meade LX200 and Autostar"
'---------------------------------------------------------------------
' Copyright  2000-2002 SPACE.com Inc., New York, NY
'
' Permission is hereby granted to use this Software for any purpose
' including combining with commercial products, creating derivative
' works, and redistribution of source or binary code, without
' limitation or consideration. Any redistributed copies of this
' Software must include the above Copyright Notice.
'
' THIS SOFTWARE IS PROVIDED "AS IS". SPACE.COM, INC. MAKES NO
' WARRANTIES REGARDING THIS SOFTWARE, EXPRESS OR IMPLIED, AS TO ITS
' SUITABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
'---------------------------------------------------------------------
'   =============
'   TELESCOPE.CLS
'   =============
'
' Implementation of ASCOM Meade telescope driver Telescope class
'
' Written:  22-Aug-00   Robert B. Denny <rdenny@dc3.com>
' Aditions: 02-Feb-02   Chris Creery <ccreery@cyanogen.com>
'           21-Apr-02   Larry Weber
'
' Edits:
'
' When      Who     What
' --------- ---     --------------------------------------------------
' 22-Aug-00 rbd     Initial edit
' 28-Aug-00 rbd     Integration testing completed. Add setup dialog.
' 14-Oct-00 rbd     Slew settling time
' 15-Oct-00 rbd     Fix Com Port registry setting name
' 13-Nov-00 rbd     Description returns String not Double, Park() does
'                   not take a command. ByVal on parameters and Lets.
'                   Allow Description when not connected.
' 14-Nov-00 rbd     TargetDeclination used HoursToHMS to format for
'                   Autostar, which barfs if there are spaces in the
'                   sexagesimal (DegreesToDMS generates spaces).
' 21-Nov-00 rbd     1.0.3 - DriverInfo is a String. Add DriverInfo
'                   code to take from project properties.
' 11-Dec-00 rbd     1.0.4 - For Autostar, increase timeout to 10 sec.
'                   Add a 1 sec delay to AbortSlew to give Autostar
'                   time to digest it. I suspect a problem with it.
'                   Remove code to stop in-progress slew when starting
'                   new one. Too slow on Autostar. Can't abort slew
'                   then start new one in Autostar anyway, it goes
'                   wild.
' 12-Dec-00 rbd     The @#$%^& Autostar's 2.0i revision broke the
'                   long format for setting target RA! And switching
'                   between short and long format in the Autostar
'                   is time-sensitive, requiring a delay. As a result,
'                   I had to change this to run the Autostar in SHORT
'                   format. ARGH!
' 13-Jan-01 rbd     Fill in missing ERR_SOURCE argument in calls to
'                   Err.Raise for "not implemented". There were many!
'                   Specific messages for not implemented.
' 21-Jan-01 rbd     New Helper.Profile object, remove Registry.bas.
'                   No more General subkey, simplify registry logic.
'                   No more Win32 here, call new Util.LoadDLL() and
'                   Util.WaitForMilliseconds()
' 25-Jan-01 rbd     New properties CanSlew, CanSlewAsync, CanSync,
'                   CanPark, CanFindHome, CanSetPark, CanUnpark
'                   Interface V1.1
' 28-Jan-00 rbd     Add description and version info display to
'                   SetupDialog box.
' 06-Feb-01 rbd     Add Tracking, CanSetTracking
' 08-Jun-01 rbd     New Profile.DeviceType to generalize Chooser and
'                   Profile.
' 08-Jun-01 rbd     Telescope 1.4: Add PulseGuide() & CanPulseGuide
' 11-Jun-01 rbd     Make it possible to call Unpark() even if never
'                   parked. Unpark() in this case is a no-op. Add
'                   code to set slow max-slew speed. This is really
'                   essential to avoid premature drive wear in
'                   automated operations.
' 05-Oct-01 rbd     1.4.2 Add ByVals where missing.
' 08-Oct-01 rbd     1.4.3 Change MSG_UNKNOWN_MEADE to refer to ASCOM
'                   web site.
' 12-Oct-01 rbd     1.4.4 Fix slew settle time > 30 sec.
' 02-Nov-01 rbd     1.4.5 Flex delimiters, for variations in Autostar
' 07-Nov-01 rbd     1.4.6 No decimal digits on minutes for Autostar
'                   (2.2 Autostar firmware rejects them)
' 05-Dec-01 rbd     1.4.7 Add Property Let for Lat/Long and UTCDate
'                   (LX200 only, Autostar does not recompute w/o
'                   handbox alignment).
' 04-May-02 rbd     1.5.1 For the LX200 GPS. This is a beast which
'                   has the characteristic of freezing for 500 or
'                   or more milliseconds every 1500 or so! Also, it
'                   doesn't implement distance bars. Rather than
'                   go through all the testing I did for the LX200
'                   classic and Autostar I, I implemented the slewing
'                   detection using RA/Dec watching for only the GPS.
'                   Some time, the distance bar logic should just be
'                   removed, using the ra/dec watching logic for all
'                   scope models. ** INCREASE SCOPE TIMEOUT TO 45
'                   SECONDS FOR AUTOSTAR II and SLOW SERIAL CAMS.
'                   Remove leading # from command strings. This was
'                   supposed to make the LX200 Classic forget about
'                   old trash in its buffer. I don't believe it is
'                   really needed, and it makes the LX200GPS deaf to
'                   commands under certain (unclear) circumstances.
' 07-May-02 rbd     1.8.2 Merge Chris Creery and Larry Weber's
'                   additions for focuser control into this. Version
'                   jumped to 1.8.x per incoming driver from Creery.
' 18-Jun-02 rbd     1.8.4 Change to out-of-proc server to support
'                    ownership of Telescope and Focuser by
'                    different processes. Add logic to prevent
'                    creating multiple instances of interfaces.
' 18-Jun-02 rbd     1.8.4 Change OpenConnection() and CloseConnection()
'                   See comments in Common module. Initial GS test
'                   now sends leading # as required by LX200 GPS
'                   moved to Common module. Change interface to Setup
'                   dialog. It knows where to store stuff now.
' 23-Jul-02 rbd     1.8.7 - Improve "Invalid Object Use" message.
' 15-Aug-02 rbd     2.0.1 - GPS now has distance bars. Remove old (and
'                   flaky) motion method of slew detection.
' 27-Sep-02 rbd     2.1.0 - Fix get_Telescope.Connected per Larry
'                   Weber. Update Min version for 1.3t.
' 04-Dec-02 rbd     2.1.1 - Remember and restore initial state of
'                   short/long mode in scope.
' 04-Jan-03 rbd     2.1.2 - Fix Get_SiderealTime() to return hours
'                   not days, operating serial timeout now 30 sec.
' 08-Mar-03 jab     get tracking now checks parked status
' 08-Mar-03 jab     get aperture, focal length, and elevation working
' 08-Mar-03 jab     slewing was uninitialized, and park flag wasn't
'                   always set right
' 09-Mar-03 jab     going to Land takes time next command was being
'                   swallowed, added 1 sec delay
' 11-Mar-03 jab     Allow elevation to default to 0 if no user input
' 12-Mar-03 jab     unpark change mode takes time, and needs a wait
' 21-Mar-03 rbd     2.2.1 - Version changed for Platform 2.2
' 17-May-03 rbd     2.2.2 - Now requires 1.6b of AutoStar II (GPS).
'                   Fix version matcher to use real last character of
'                   MIN_VER_STRING instead of always 'o'.
'                   Shorten extra wait to 1 sec got GPS. It seems to
'                   give a proper indication with the dist. bars.
' 06-Jun-03 rbd     2.2.3 - Suspend settle time when parking Classic.
' 24-Jun-03 rbd     2.3.1 - Version change for Platform 2.3
' 09-Aug-03 rbd     2.3.2 - Parking Classic for southern hemi
' 03-Sep-03 rbd     3.0.1 - Version for Platform 3.0, try long format
'                   on Autostar I, more parking
' 09-Sep-03 rbd     3.0.2 - Long format on Autostar 1
' 28-sep-03 jab     added control of auto unpark
' 25-Oct-o3 jab     setup dialog now works while connected
' 03-Jan-04 jab     convert all Alt/Az coordinates based on polar or alt/az
'                   mounting prior to sending through as target coords
' 20-Oct 20 jab     Use the undocumented Mgxddd command for GPS pulseguide
' 23-Nov-04 rbd     4.0.1 - Version change for Platform 4.0
' 03-Jan-05 rbd     4.1.0 - New LX200GPS startup code for connecting
'                   to parked scope. Move buffer-clear on startup to
'                   AFTER sending the "butt-kick" to GPS. Add wait
'                   loop for "I" in Unpark(), new reboot-fail error.
' 22-Feb-05 rbd     4.1.1 - Skip Auto Set Time for Autostar 495/497
'---------------------------------------------------------------------

Option Explicit

Private Const MIN_VERSION As Single = 2#
Private Const MIN_VER_STRING As String = "2.0i"
Private Const SIDRATE As Long = 0.9972695677   ' synodic/solar seconds per sidereal second

' conversions
Private Const DEG_RAD As Double = 0.0174532925
Private Const RAD_DEG As Double = 57.2957795
Private Const HRS_RAD As Double = 0.2617993881
Private Const RAD_HRS As Double = 3.81971863

Private Const INVALID_PARAMETER As Double = -10000#

Public Enum AlignmentModes
    algAltAz = 0
    algPolar = 1
    algGermanPolar = 2
End Enum

Public Enum DriveRates
    driveSidereal = 0
    driveLunar = 1
    driveSolar = 2
    driveKing = 3
End Enum

Public Enum EquatorialCoordinateType
    equOther = 0
    equLocalTopocentric = 1
    equJ2000 = 2
    equJ2050 = 3
    equB1950 = 4
End Enum

Public Enum GuideDirections
    guideNorth = 0
    guideSouth = 1
    guideEast = 2
    guideWest = 3
End Enum

Public Enum PierSide
    pierEast = 0
    pierWest = 1
End Enum

Public Enum TelescopeAxes
    axisPrimary = 0
    axisSecondary = 1
    axisTertiary = 2
End Enum

Private Enum ScopeModel
    modelUnknown = -1
    modelSimple = 0         ' for testing, LX200 emulators, etc.
    modelClassic = 1        ' original LX200's
    modelClassic16 = 2      ' original 16"
    modelClassicGEM = 3     ' 650/750 GEM
    modelAutostar = 4       ' autostar 1
    modelAutostarGEM = 5    ' autostar 1 with GEM
    modelGPS = 6            ' LX200GPS
End Enum

Private m_bTestCoord As Boolean
Private m_eModel As ScopeModel
Private m_bAsleep As Boolean

Private m_AxisRates As AxisRates
Private m_TrackingRates As TrackingRates

Private m_eTrackingRate As DriveRates
Private m_eAlignMode As AlignmentModes
Private m_bSlewing As Boolean
Private m_bTracking As Boolean
Private m_bInvalidCoords As Boolean
Private m_bAbortSlew As Boolean
Private m_bParked As Boolean
Private m_lSlewStartTix As Long
Private m_lAsyncEndTix As Long
Private m_iRaBar As Integer
Private m_iDecBar As Integer
Private m_bLongFormat As Boolean
Private m_bOrigLongFormat As Boolean
Private m_dDecRate As Double
Private m_dRARate As Double
Private m_bOffsetTracking As Boolean

Private m_sRADelimHrs As String
Private m_sRADelimMin As String
Private m_sRADelimSec As String
Private m_sDecDelimDeg As String
Private m_sDecDelimMin As String
Private m_sDecDelimSec As String
Private m_sAzDelimDeg As String
Private m_sAzDelimMin As String
Private m_sAzDelimSec As String

Private m_Util As DriverHelper.Util
Private m_Profile As DriverHelper.Profile
Private m_iSettleTime As Integer

Private m_dAlt As Double
Private m_dAz As Double
Private m_dRA As Double
Private m_dDec As Double
Private m_dTargetRA As Double   ' monitored for read before write
Private m_dTargetDec As Double  ' monitored for read before write
Private m_dLat As Double        ' cached since we use it a lot
Private m_dGuideRate As Double  ' used for both Dec and RA
'
' Locally used Win32 API
'
Private Declare Function GetTickCount Lib "kernel32" () As Long

Private Sub Class_Initialize()

    Dim buf As String

    If g_bTelescopeCreated Then _
        Err.Raise 425, App.Title, "Invalid object use: Only one program can use this Telescope."  ' Prevent multiple use
    g_bTelescopeCreated = True
    
    Set m_Util = New DriverHelper.Util
    Set m_Profile = New DriverHelper.Profile
    m_Profile.DeviceType = "Telescope"              ' We're a Telescope driver
    m_Profile.Register SCOPE_ID, DESC
    
    On Error Resume Next
    m_Profile.DeleteSubKey SCOPE_ID, "General"      ' Delete fossil early beta stuff
    On Error GoTo 0
    
    ' debugging output value
    m_Profile.WriteValue SCOPE_ID, "BreadCrumb", "starting up"

    ' Assure this is loaded and available
    m_Util.LoadDLL App.Path & "\astro32.dll"
    
    ' set up for pure coordinate fetching test mode
    buf = m_Profile.GetValue(SCOPE_ID, "TestCoord")
    If buf = "" Then _
        buf = "False"       ' initialize to normal mode, if never been set
    m_bTestCoord = CBool(buf)
    ' write the value so hand edits of the registry can change behavior
    m_Profile.WriteValue SCOPE_ID, "TestCoord", CStr(m_bTestCoord)
    
    m_iSettleTime = 0
    m_bParked = False
    m_bSlewing = False
    m_bTracking = False
    m_bInvalidCoords = False
    m_bAsleep = False
    m_dRARate = 0#
    m_dDecRate = 0#
    m_bOffsetTracking = False
    
End Sub

Private Sub Class_Terminate()
    
    On Error Resume Next
    Me.Connected = False                            ' Force disconnect
    Set g_Scope = Nothing
    g_bTelescopeCreated = False
    
End Sub

Public Property Get AlignmentMode() As AlignmentModes

    CheckConnected False, m_bTracking, False
    
    AlignmentMode = m_eAlignMode
    
End Property

Public Property Get Altitude() As Double
    
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim dec As Double
        
    CheckConnected False, m_bTracking, True
    
    ' if scope is indeterminate, return previous value
    If m_bInvalidCoords Then
        Altitude = m_dAlt
        Exit Property
    End If
        
    Alt = m_Util.DMSToDegrees(Me.CommandString("GA"))
    
    If m_bTracking Or m_bTestCoord Then
        Altitude = Alt
    Else
        If m_eAlignMode = algAltAz Then
            Altitude = Alt
        Else
            Az = m_Util.DMSToDegrees(Me.CommandString("GZ"))
            
            ' Wrap it around 180 deg to the "real" azimuth.
            If m_eModel <> modelGPS Then _
                Az = RangeAz(Az + 180#)
            
            ha = RangeHA((Az - 180#) / 15#)
            dec = Alt
            calc_altaz -ha, dec, m_dLat, Az, Alt
            Altitude = Alt
        End If
    End If
    
    m_dAlt = Altitude

End Property

Public Property Get ApertureArea() As Double

    Dim buf As String
    Dim testVal As Double
    
    buf = m_Profile.GetValue(SCOPE_ID, "ApertureArea")
    
    ' make sure our error gets thrown, not val's
    On Error Resume Next
        testVal = -1
        If buf <> "" Then _
            testVal = val(buf)
    On Error GoTo 0

    If testVal <= 0 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Property ApertureArea" & MSG_PROP_RANGE_ERROR
                
    ApertureArea = testVal
            
End Property

Public Property Get ApertureDiameter() As Double

    Dim buf As String
    Dim testVal As Double
    
    buf = m_Profile.GetValue(SCOPE_ID, "Aperture")
    
    ' make sure our error gets thrown, not CDbl's
    On Error Resume Next
        testVal = -1
        If buf <> "" Then _
            testVal = val(buf)
    On Error GoTo 0

    If testVal <= 0 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Property ApertureDiameter" & MSG_PROP_RANGE_ERROR
                
    ApertureDiameter = testVal

End Property

Public Property Get AtHome() As Boolean

    AtHome = False
    
End Property

Public Property Get AtPark() As Boolean
   
    CheckConnected False, m_bTracking, False
    
    AtPark = m_bParked
    
End Property

Public Property Get Azimuth() As Double
    
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim dec As Double
    
    CheckConnected False, m_bTracking, True
    
    ' if scope is indeterminate, return previous value
    If m_bInvalidCoords Then
        Azimuth = m_dAz
        Exit Property
    End If
    
    Az = m_Util.DMSToDegrees(Me.CommandString("GZ"))

    ' Wrap it around 180 deg to the "real" azimuth.
    If m_eModel <> modelGPS Then _
        Az = RangeAz(Az + 180#)
    
    If m_bTracking Or m_bTestCoord Then
        Azimuth = Az
    Else
        If m_eAlignMode = algAltAz Then
            Azimuth = Az
        Else
            Alt = m_Util.DMSToDegrees(Me.CommandString("GA"))
            ha = RangeHA((Az - 180#) / 15#)
            dec = Alt
            calc_altaz -ha, dec, m_dLat, Az, Alt
            Azimuth = Az
        End If
    End If
    
    m_dAz = Azimuth
    
End Property

Public Property Get CanFindHome() As Boolean

    CanFindHome = False

End Property

Public Property Get CanPark() As Boolean
    
    CanPark = True
    
End Property

Public Property Get CanPulseGuide() As Boolean

    CanPulseGuide = True
    
End Property

Public Property Get CanSetDeclinationRate() As Boolean
    
    CanSetDeclinationRate = False

End Property

Public Property Get CanSetGuideRates() As Boolean

    CheckConnected False, m_bTracking, False
    
    ' claims to work, more testing needed before we declare victory ???
    CanSetGuideRates = (m_eModel = modelGPS)

End Property

Public Property Get CanSetPark() As Boolean

    CheckConnected False, m_bTracking, False
    
    ' The only scope known to support the set park code is the
    ' classic LX200 in the north hemisphere (others should be checked before allowing ???)
    If m_eModel = modelClassic Then
        CanSetPark = IIf(m_dLat < 0, False, True)
    Else
        CanSetPark = False
    End If

End Property

Public Property Get CanSetPierSide() As Boolean

    CanSetPierSide = False
    
End Property

Public Property Get CanSetRightAscensionRate() As Boolean
    
    CanSetRightAscensionRate = False

End Property


Public Property Get CanSetTracking() As Boolean
    
    CheckConnected False, m_bTracking, False
    
    ' GPS may be supportable ??? (failed, on recent Meade firmware, dissabled for now)
    CanSetTracking = (m_eModel = modelClassic) Or (m_eModel = modelClassic16)

End Property

Public Property Get CanSlew() As Boolean

    CanSlew = True
    
End Property

Public Property Get CanSlewAltAz() As Boolean
    
    CanSlewAltAz = True
        
End Property

Public Property Get CanSlewAltAzAsync() As Boolean
    
    CanSlewAltAzAsync = True
        
End Property

Public Property Get CanSlewAsync() As Boolean
    
    CanSlewAsync = True
    
End Property

Public Property Get CanSync() As Boolean

    CanSync = True
    
End Property

Public Property Get CanSyncAltAz() As Boolean

    CanSyncAltAz = False
    
End Property

Public Property Get CanUnpark() As Boolean

    CheckConnected False, m_bTracking, False
    
    ' Autostar 49x cannot be unparked
    CanUnpark = (m_eModel = modelClassic) Or _
        (m_eModel = modelClassic16) Or _
        (m_eModel = modelGPS)
    
End Property

Public Property Get Connected() As Boolean

    Connected = g_bTelescopeConnected
    
End Property

Public Property Let Connected(ByVal newVal As Boolean)
    Dim buf As String, sRA As String, sDec As String, sAz As String
    Dim rx As RegExp
    Dim mt As MatchCollection
    Dim tmo As Long
    Dim ver As Single, rev As String
    Dim Alt As Double
    Dim Az As Double
    Dim CurDate As Double
    Dim GpsParked As Boolean
    Dim I As Integer

    
    ' don't try to change state if already there
    If newVal = g_bTelescopeConnected Then _
        Exit Property
        
    m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "In Connect"
    If newVal Then                                      ' CONNECTING
            
        Set rx = New RegExp                             ' We use this in a couple of places
        
        '
        ' (1) Set up the communications link. Default to COM1, set 2 sec. timeout
        '
        OpenConnection devTelescope                     ' Open port and setup serial object
        If g_Scope Is Nothing Then Set g_Scope = Me
        '
        ' (2)
        '
        If LCase$(m_Profile.GetValue(SCOPE_ID, "GPSParked")) = "true" Then
            '
            ' (2a) If the last detected model was an LX200GPS, and if it was parked by
            '      this driver, issue an "I" command to reboot it and wait till it
            '      wakes up. It should be alive within 60 sec. Failure means no Meade
            '      scope connected, as we use the "GS" command to poll for the reboot.
            '
            m_Profile.WriteValue SCOPE_ID, "GPSParked", "" ' This is a fragile state
            m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "Rebooting GPS"
            g_SharedSerial.Transmit ":I#"               ' Reboot the parked LX200GPS
            m_Util.WaitForMilliseconds 5000             ' Wait at least 5 sec
            On Error Resume Next
            For I = 1 To 11                             ' For up to 55 sec more
                g_SharedSerial.ClearBuffers
                g_SharedSerial.Transmit ":GS#"          ' Try for the Sidereal Time
                Call g_SharedSerial.ReceiveTerminated("#")  ' Timeouts are 2 sec
                If Err.Number = 0 Then Exit For         ' Got it!
                Err.Clear
                m_Util.WaitForMilliseconds 3000         ' Wait another 3 sec, for a total of 5 sec.
            Next I
            On Error GoTo 0
            If I > 11 Then                              ' Scope never responded, prolly no scope!
                CloseConnection devTelescope
                Err.Raise SCODE_GPSREBOOT_FAIL, ERR_SOURCE, _
                    "Connecting - " & MSG_GPSREBOOT_FAIL
            End If
            m_bAsleep = False
            m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "Reboot GPS OK"
        Else
            '
            ' (2b) Other scope or non-parked GPS. Just poll for sidereal time to
            '      verify that some sort of Meade scope is there.
            '
            m_Profile.WriteValue SCOPE_ID, "GPSParked", "" ' This is a fragile state
            m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "Detect Meade scope"
            g_SharedSerial.Transmit ":GS#"              ' Now try for the Sidereal Time
            On Error Resume Next
            Call g_SharedSerial.ReceiveTerminated("#")
            If Err.Number <> 0 Then                     ' No response
                CloseConnection devTelescope
                Err.Raise SCODE_NO_SCOPE, ERR_SOURCE, MSG_NO_SCOPE
            End If
        End If
        m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "connection open"
        '
        ' (3) Check to see if we're configured for the LX-100 16",
        '     which has some quirks we need to handle. If it has never
        '     been configured at all, then require that!
        '
        buf = m_Profile.GetValue(SCOPE_ID, "LX200 16 Inch")
        If buf = "" Then                                ' Never configured. Require setup
            CloseConnection devTelescope                ' Release port for telescope
            Err.Raise SCODE_NEED_SETUP, ERR_SOURCE, MSG_NEED_SETUP
        Else
            If CBool(buf) Then
                m_eModel = modelClassic16
            Else
                m_eModel = modelClassic
            End If
        End If
        '
        ' (4) Try a version handshake. This works only on the Autostar I/II.
        '     Depending on the response (or lack thereof), determine the
        '     model of LX200. If it's a GPS, require a "recent enough" version
        '     of the firmware. Receive timeout is still 2 seconds
        '
        On Error GoTo NO_VER                            ' -- error trap --
            buf = Me.CommandString("GVF")               ' Read version
        On Error GoTo 0                                 ' -- end trap --
        
        If InStr(buf, "Autostar") > 0 Then              ' Is it an Autostar 1?
            m_eModel = modelAutostar
        ElseIf Left$(buf, 7) = "Version" Then           ' LX200 GPS
            m_eModel = modelGPS
            
            On Error GoTo NO_VER                        ' At least TRY this stuff
                buf = Mid$(buf, 8)                      ' Version if any (n.nx)
                rx.Global = False                       ' Split number & letter
                rx.Pattern = "([0-9]*\.[0-9]*)([a-z])"
                Set mt = rx.Execute(buf)
                ver = CSng(mt(0).SubMatches(0))         ' Isolated version number
                rev = mt(0).SubMatches(1)               ' Revision letter
            On Error GoTo 0
            
            If ver < MIN_VERSION Or _
                (ver = MIN_VERSION And Asc(rev) < Asc(Right$(MIN_VER_STRING, 1))) Then
                MsgBox "Your Autostar II firmware (" & buf & ") should be updated to " & _
                        MIN_VER_STRING & " or later. Some things may not work.", _
                        (vbOKOnly + vbExclamation + vbMsgBoxSetForeground), _
                        ERR_SOURCE
                '** Allow connection ???
                'CloseConnection devTelescope
                'Exit Property                           ' CONNECT FAILED!
            End If
        Else
            m_eModel = modelSimple  ' allow this once we get simple working ???
            CloseConnection devTelescope                ' Release port for telescope
            Err.Raise SCODE_UNKNOWN_MEADE, ERR_SOURCE, MSG_UNKNOWN_MEADE
        End If
        GoTo VER_DONE
NO_VER:
        Resume VER_DONE
VER_DONE:
        g_SharedSerial.ClearBuffers
        m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "version done"
       
        ' (5) Get the alignment mode if possible. No go on old ROMs.
        '
        m_bParked = False                               ' for now, update later
        m_bTracking = True                              ' for now, update later
        
        g_SharedSerial.ReceiveTimeout = 2                     ' Allow 2 sec
        ' control-f in hyperterm will work !!!
        g_SharedSerial.Transmit Chr$(6)
        On Error GoTo NO_ALG                            ' -- error trap ---
        buf = g_SharedSerial.ReceiveCounted(1)                ' Receive one character
        On Error GoTo 0                                 ' -- end trap ---
        
        Select Case buf
            Case "L"                                    ' LAND mode (parked?)
                m_bTracking = False
                buf = m_Profile.GetValue(SCOPE_ID, "AlignMode", "")
                If buf = "" Then
                    ' bail out, we don't know how to turn tracking on
                    MsgBox "Your telescope need to be tracking" & vbCrLf & _
                        " the first time this ASCOM driver connects.", _
                        (vbOKOnly + vbExclamation + vbMsgBoxSetForeground), _
                        ERR_SOURCE
                    CloseConnection devTelescope
                    Err.Raise SCODE_LAND_MODE, ERR_SOURCE, MSG_LAND_MODE
                End If
                m_eAlignMode = val(buf)
            Case "A"
                m_eAlignMode = algAltAz
            Case "P"
                m_eAlignMode = algPolar
            Case "G"
                If m_eModel = modelAutostar Then
                    m_eModel = modelAutostarGEM
                Else
                    m_eModel = modelClassicGEM
                End If
                m_eAlignMode = algGermanPolar
                ' may not support land mode switches for park amd track control ???
                ' G mode not setable programatically !!!
            Case Else                                   ' Not an LX200? model?
                CloseConnection devTelescope            ' Release port for telescope
                Err.Raise SCODE_NOT_MEADE, ERR_SOURCE, MSG_NOT_MEADE
        End Select
        GoTo ALG_DONE
        
NO_ALG:
        ' could go to simple mode instead ???
        CloseConnection devTelescope
        Err.Raise SCODE_NOT_MEADE, ERR_SOURCE, MSG_NOT_MEADE
        Resume ALG_DONE
        
ALG_DONE:
        g_SharedSerial.ClearBuffers           ' Toss any junk remaining in buffers
        m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "alignment mode done"
        
        '
        ' (6) Set slow slew speed. This is ESSENTIAL to preserve the drives
        '     when using in "industrial strength" applications. Not supported
        '     on the Autostar I.
        '
        If Not m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then
            If m_eModel = modelGPS Then
                Me.CommandBool "Sw 5"                   ' Speed 5 for new GPS
            Else
                Me.CommandBool "Sw 3"                   ' Speed 3 for Classic
            End If
        End If
        '
        ' (7) Change to 30 sec timeouts and assure Long Format mode
        '
        g_SharedSerial.ReceiveTimeout = 30              ' Switch to 30 sec timeout
        m_bOrigLongFormat = IsLongFormat()              ' Remember initial format
        SetLongFormat True                              ' Switch to long
        '
        ' (8) Get the sexagesimal delimiters that this implementation uses
        '
        sRA = Me.CommandString("GR")
        m_Util.WaitForMilliseconds 200
        sDec = Me.CommandString("GD")
        m_Util.WaitForMilliseconds 200
        sAz = Me.CommandString("GZ")
        
        Set rx = New RegExp
        rx.Global = True
        rx.Pattern = "[\+\-]*[0-9\.]+([^0-9\.]+)[0-9\.]*([^0-9\.]*)[0-9\.]*([^0-9\.]*)"
        
        Set mt = rx.Execute(sRA)
        m_sRADelimHrs = mt(0).SubMatches(0)
        m_sRADelimMin = mt(0).SubMatches(1)
        m_sRADelimSec = mt(0).SubMatches(2)
        
        Set mt = rx.Execute(sDec)
        m_sDecDelimDeg = mt(0).SubMatches(0)
        m_sDecDelimMin = mt(0).SubMatches(1)
        m_sDecDelimSec = mt(0).SubMatches(2)
        
        Set mt = rx.Execute(sAz)    ' apparently Az is different than Dec (jab)
        m_sAzDelimDeg = mt(0).SubMatches(0)
        m_sAzDelimMin = mt(0).SubMatches(1)
        m_sAzDelimSec = mt(0).SubMatches(2)
        '
        ' (9) Set up rates collections and initialize rate
        '
        Set m_AxisRates = New AxisRates            ' empty for when no axis support
        Set m_TrackingRates = New TrackingRates
        m_TrackingRates.Add driveSidereal
        
        ' The GPS claim more support, but these commands just do not work (yet)
'        If m_eModel = modelGPS Then
'            m_TrackingRates.Add driveLunar
'            m_TrackingRates.Add driveSolar
'        End If
        If m_eModel = modelGPS Then
            Me.GuideRateRightAscension = 0#     ' Dec rate ties to RA rate
        Else
            m_dGuideRate = 0#
        End If
        Me.TrackingRate = driveSidereal         ' set a default
        '
        ' (10) Optionally set time from PC (Not on any autostar, Classic only)
        '      WARNING - TESTS FOR MODEL ALSO IN Property Let UTCDate
        '
        If m_eModel <> modelAutostar And m_eModel <> modelAutostarGEM And m_eModel <> modelGPS Then
            buf = m_Profile.GetValue(SCOPE_ID, "AutoSetTime")
            If buf = "1" Then
                CurDate = CDbl(Now()) + (CDbl(utc_offs()) / 86400#)
                Me.UTCDate = CurDate
            End If
        End If
        '
        ' (11) Buffer the scope's latitude (for crazy parked stuff)
        '
        m_dLat = Me.SiteLatitude                ' get a global version of latitude
        '
        ' (12) Deal with the parked / tracking states
        '
        On Error Resume Next
            If m_bTracking Then
                m_Profile.WriteValue SCOPE_ID, "AlignMode", CStr(m_eAlignMode)
            Else
                m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "calling unsetpark"
                Unsetpark           ' will resync position, if needed, and set park state
                m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "back from unsetpark"
            End If
        On Error GoTo 0
        '
        ' (13) Seed the coordinates for possible indeterminate state later
        '
        m_bInvalidCoords = False
        m_dAlt = Me.Altitude
        m_dAz = Me.Azimuth
        m_dDec = Me.Declination
        m_dRA = Me.RightAscension
        m_dTargetRA = INVALID_PARAMETER
        m_dTargetDec = INVALID_PARAMETER
        
        m_Profile.WriteValue SCOPE_ID, "BreadCrumbC", "done with connect"
        
    Else                                                ' DISCONNECTING
        On Error Resume Next                            ' Best efforts...
        If g_SharedSerial.Connected Then
            If m_eModel <> modelAutostar And m_eModel <> modelAutostarGEM Then
                SetLongFormat m_bOrigLongFormat
            End If
        End If
        CloseConnection devTelescope                    ' Release port for telescope
    End If

    Exit Property
    
NO_SCOPE:
    CloseConnection devTelescope                        ' Release port for telescope
    Err.Raise SCODE_NO_SCOPE, ERR_SOURCE, MSG_NO_SCOPE

End Property

Public Property Get Declination() As Double

    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim dec As Double
    
    CheckConnected False, m_bTracking, True
    
    ' if scope is indeterminate, return previous value
    If m_bInvalidCoords Then
        Declination = m_dDec
        Exit Property
    End If
    
    If m_bTracking Or m_bTestCoord Then
        Declination = m_Util.DMSToDegrees(Me.CommandString("GD"))
    Else
        Alt = m_Util.DMSToDegrees(Me.CommandString("GA"))
        
        If m_eAlignMode = algAltAz Then
            Az = m_Util.DMSToDegrees(Me.CommandString("GZ"))
            
            ' Wrap Az around 180 deg to the "real" azimuth.
            If m_eModel <> modelGPS Then _
                Az = RangeAz(Az + 180#)
            
            calc_hadec Az, Alt, m_dLat, ha, dec
        Else
            dec = Alt
        End If
        
        Declination = dec
    End If
    
    m_dDec = Declination
    
End Property

Public Property Get DeclinationRate() As Double
    
    DeclinationRate = 0#
    
End Property

Public Property Let DeclinationRate(ByVal newVal As Double)
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Property DeclinationRate" & MSG_NOT_IMPLEMENTED
        
End Property

Public Property Get Description() As String

    '
    ' When connected, return real name of scope
    ' as we detected. Otherwise, return a
    ' generic name.
    '
    Description = "All Meade Telescopes"
    If g_SharedSerial.Connected Then
        If m_eModel = modelAutostar Then
            Description = CommandString("GVF")
        ElseIf m_eModel = modelGPS Then
            Description = "LX200 GPS " & CommandString("GVF")
        ElseIf m_eModel = modelClassic16 Then
            Description = "Meade LX200 Classic 16 inch"
        ElseIf m_eModel = modelClassic Then
            Description = "Meade LX200 Classic"
        ElseIf m_eModel = modelClassicGEM Then
            Description = "Meade GEM"
        ElseIf m_eModel = modelAutostarGEM Then
            Description = "Meade Autostar GEM"
        End If
    End If
    
End Property

Public Property Get DoesRefraction() As Boolean
    
    DoesRefraction = True
            
End Property

Public Property Let DoesRefraction(ByVal newVal As Boolean)
        
    If Not newVal Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property DoesRefraction" & MSG_PROP_RANGE_ERROR
            
End Property

Public Property Get DriverInfo() As String
    '
    ' Use the Project/Properties sheet, Make tab, to set these
    ' items. That way they will show in the Version tab of the
    ' Explorer property sheet, and the exact same data will
    ' show in Telescope.DriverInfo.
    '
    DriverInfo = App.FileDescription & " " & _
                App.Major & "." & App.Minor & "." & App.Revision
    If App.CompanyName <> "" Then _
        DriverInfo = DriverInfo & vbCrLf & App.CompanyName
    If App.LegalCopyright <> "" Then _
        DriverInfo = DriverInfo & vbCrLf & App.LegalCopyright
    If App.Comments <> "" Then _
        DriverInfo = DriverInfo & vbCrLf & App.Comments
    
End Property

Public Property Get DriverVersion() As String
    
    DriverVersion = App.Major & "." & App.Minor
        
End Property

Public Property Get EquatorialSystem() As EquatorialCoordinateType
      
    EquatorialSystem = equLocalTopocentric
    
End Property

Public Property Get FocalLength() As Double

    Dim buf As String
    Dim testVal As Double
    
    buf = m_Profile.GetValue(SCOPE_ID, "FocalLength")
    
    ' make sure our error gets thrown, not CDbl's
    On Error Resume Next
        testVal = -1
        If buf <> "" Then _
            testVal = val(buf)
    On Error GoTo 0
    
    If testVal <= 0 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Property FocalLength" & MSG_PROP_RANGE_ERROR
                
    FocalLength = testVal

End Property

Public Property Get GuideRateDeclination() As Double

    GuideRateDeclination = Me.GuideRateRightAscension

End Property

Public Property Let GuideRateDeclination(ByVal newVal As Double)
                    
    Me.GuideRateRightAscension = newVal
    
End Property

Public Property Get GuideRateRightAscension() As Double

    CheckConnected False, m_bTracking, False

    GuideRateRightAscension = m_dGuideRate

End Property

Public Property Let GuideRateRightAscension(ByVal newVal As Double)

    Dim asVal As Double     ' newVal in arc sec
    
    CheckConnected False, m_bTracking, True
    
    asVal = newVal * 3600#
    
    ' The range check below is from a Meade document
    If m_eModel <> modelGPS Or asVal > 15.0417 Or asVal < 0# Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property GuideRate" & MSG_PROP_RANGE_ERROR
    
    ' untested but syntax verified on lx200gps group ???
    ' setup -> telescope -> guiderate should show a percentage that changes on handset
    Me.CommandBlind "Rg" & Format$(asVal, "00.0")
    
    m_dGuideRate = newVal
    
End Property

Public Property Get InterfaceVersion() As Integer

    InterfaceVersion = 2
        
End Property

Public Property Get IsPulseGuiding() As Boolean
      
    CheckConnected False, m_bTracking, False
    
    ' pulse guiding is synchronous for this driver
    IsPulseGuiding = False
        
End Property

Public Property Get Name() As String
    '
    ' When connected, return real name of scope
    ' as we detected. Otherwise, return a
    ' generic name.
    '
    Name = "Meade"
    If g_SharedSerial.Connected Then
        If m_eModel = modelAutostar Then
            Name = "Autostar I"
        ElseIf m_eModel = modelGPS Then
            Name = "Autostar II"
        ElseIf m_eModel = modelClassic16 Then
            Name = "LX200 16"""
        ElseIf m_eModel = modelClassic Then
            Name = "LX200"
        ElseIf m_eModel = modelClassicGEM Then
            Name = "GEM"
        ElseIf m_eModel = modelAutostarGEM Then
            Name = "Autostar GEM"
        End If
    End If

End Property

Public Property Get RightAscension() As Double
    
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim RA As Double
    Dim dec As Double
    Dim ST As Double
    
    CheckConnected False, m_bTracking, True
    
    ' if scope is indeterminate, return previous value
    If m_bInvalidCoords Then
        RightAscension = m_dRA
        Exit Property
    End If
    
    If m_bTracking Or m_bTestCoord Then
        RightAscension = m_Util.HMSToHours(Me.CommandString("GR"))
    Else
        ST = SiderealTime
        Az = m_Util.DMSToDegrees(Me.CommandString("GZ"))
        
        ' Wrap Az around 180 deg to the "real" azimuth.
        If m_eModel <> modelGPS Then _
            Az = RangeAz(Az + 180#)
        
        If m_eAlignMode = algAltAz Then
            Alt = m_Util.DMSToDegrees(Me.CommandString("GA"))
            calc_hadec Az, Alt, m_dLat, ha, dec
            RA = RangeRA(ST - ha)
        Else
            RA = RangeRA(ST - ((Az - 180#) / 15#))
        End If
        
        RightAscension = RA
    End If
    
    m_dRA = RightAscension
    
End Property

Public Property Get RightAscensionRate() As Double
    
    RightAscensionRate = 0#

End Property

Public Property Let RightAscensionRate(ByVal newVal As Double)
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Property RightAscensionRate" & MSG_NOT_IMPLEMENTED
       
End Property

Public Property Get SideOfPier() As PierSide
    
    CheckConnected False, m_bTracking, False
    
    ' Meade GEMs might be possible to model in the future ???
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Property SideOfPier" & MSG_NOT_IMPLEMENTED
        
End Property

Public Property Let SideOfPier(ByVal newVal As PierSide)
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Property SideOfPier" & MSG_NOT_IMPLEMENTED
        
    ' but if we do ever implement, its defined as asynchronous
    
End Property

Public Property Get SiderealTime() As Double

    CheckConnected False, m_bTracking, True
    
    SiderealTime = CDbl(CDate(Me.CommandString("GS"))) * 24#

End Property

Public Property Get SiteElevation() As Double

    Dim buf As String
    Dim elev As Double
    
    buf = m_Profile.GetValue(SCOPE_ID, "SiteElevation")
    
    ' if not set at all, return 0
    If buf = "" Then
        elev = 0
    Else
        On Error Resume Next
            elev = -1000000    ' illigal number to catch failed convertion
            elev = val(buf)   ' don't allow conversion to error out
        On Error GoTo 0
        
        If elev = -1000000 Then _
            Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Property Elevation" & MSG_PROP_RANGE_ERROR
    End If
    
    SiteElevation = elev

End Property

Public Property Let SiteElevation(ByVal newVal As Double)

    If newVal < -300 Or newVal > 10000 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property SiteElevation " & MSG_PROP_RANGE_ERROR
            
    m_Profile.WriteValue SCOPE_ID, "SiteElevation", Str(newVal)

End Property

Public Property Get SiteLatitude() As Double

    CheckConnected False, m_bTracking, True
    
    SiteLatitude = m_Util.DMSToDegrees(Me.CommandString("Gt"))
    
    m_dLat = SiteLatitude   ' refresh the global, we could have just returned
                            ' the global, but this is more robust
    
End Property

Public Property Let SiteLatitude(ByVal newVal As Double)

    CheckConnected False, m_bTracking, True
    
    ' Autostar cannot recalculate its transform after alignment,
    ' thus changing the site location or time will invalidate its
    ' internal model. Disallow on Autostar I and II.
    '
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Or m_eModel = modelGPS Then
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
                "Property Let SiteLatitude (Autostar)" & MSG_NOT_IMPLEMENTED
    End If
    
    If newVal < -90 Or newVal > 90 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property SiteLatitude " & MSG_PROP_RANGE_ERROR
            
    If Not Me.CommandBool("St " & DegtoDM(newVal, 2)) Then
        Err.Raise SCODE_BAD_LATITUDE, ERR_SOURCE, MSG_BAD_LATITUDE
    End If
    
    m_dLat = newVal ' refresh the global
    
End Property

Public Property Get SiteLongitude() As Double

    CheckConnected False, m_bTracking, True
    '
    ' Convert from 0-359 westward to +/- 180 +E
    '
    SiteLongitude = -m_Util.DMSToDegrees(Me.CommandString("Gg"))
    If SiteLongitude < -180# Then _
        SiteLongitude = SiteLongitude + 360#

End Property

Public Property Let SiteLongitude(ByVal newVal As Double)

    CheckConnected False, m_bTracking, True
    
    ' Autostar cannot recalculate its transform after alignment,
    ' thus changing the site location or time will invalidate its
    ' internal model. Disallow on Autostar.
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Or m_eModel = modelGPS Then
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
                "Property Let SiteLongitude (Autostar)" & MSG_NOT_IMPLEMENTED
    End If
    
    If newVal < -180 Or newVal > 180 Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property SiteLongitude " & MSG_PROP_RANGE_ERROR
    
    ' pre-GPS invert sign
    If m_eModel <> modelGPS Then _
        newVal = -newVal

    ' Convert from +/- 180 (ASCOM spec) to 0-360 (Meade spec)
    If newVal < 0 Then _
        newVal = 360# + newVal
        
    If Not Me.CommandBool("Sg " & DegtoDM(newVal, 3)) Then
        Err.Raise SCODE_BAD_LONGITUDE, ERR_SOURCE, MSG_BAD_LONGITUDE
    End If

End Property

Public Property Get Slewing() As Boolean
    Dim buf As String, mvg As Boolean, tmo As Integer
    Dim rb As Integer, db As Integer, RA As Double, dec As Double
    
    CheckConnected False, m_bTracking, False
    
    ' This has a dual purpose: It not only returns to the caller
    ' the slewing status, but it also updates the end-interval
    ' ticks and implements the completion interval stuff. The
    ' m_bSlewing flag is not cleared until a set time after the
    ' scope distance bars indicate that the slew has completed.
    '
    If Not m_bSlewing Then                          ' Short-circuit when not slewing
        Slewing = False
        Exit Property                               ' No slew now
    End If
    
    ' only allow if scope is awake
    If m_bAsleep Then _
        Err.Raise SCODE_ASLEEP, ERR_SOURCE, MSG_ASLEEP
        
    buf = Trim$(Me.CommandString("D"))              ' Get distance bars
    If (buf <> "") Then                             ' Moving
        mvg = True
    Else                                            ' Not moving
        mvg = False
    End If
    
    If (Not m_bAbortSlew) And mvg Then
        m_lAsyncEndTix = GetTickCount() + (CLng(m_iSettleTime) * 1000&)
        If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then
            m_lAsyncEndTix = m_lAsyncEndTix + 11500         ' Start with a minimum of 11.5 sec
        ElseIf m_eModel = modelGPS Then
            m_lAsyncEndTix = m_lAsyncEndTix + 1000          ' Start with a minimum of 1 sec
        Else
            m_lAsyncEndTix = m_lAsyncEndTix + 4500          ' Start with a minimum of 4.5 sec
        End If
        '
        ' If this is a 16", we test the distance bars and try to
        ' decide if the slew failed (even through it reported "0"
        ' above). This is ugly and possibly quirky, so do it only
        ' for the 16".
        '
        If m_eModel = modelClassic16 Then
            rb = Len(Trim$(Left$(buf, 16)))
            db = Len(Trim$(Right$(buf, 16)))
            If ((GetTickCount() - m_lSlewStartTix) > 30000) And _
                (rb = m_iRaBar) And (db = m_iDecBar) Then
                Me.AbortSlew                            ' Slew fail on 16"
                Err.Raise SCODE_SLEW_16FAIL, ERR_SOURCE, MSG_SLEW_16FAIL
            End If
        End If
        Slewing = True
    Else
        '
        ' This implements the trailing 4/11 seconds to allow vernier
        ' (skip it on a slew abort)
        '
        If (Not m_bAbortSlew) And (GetTickCount() < m_lAsyncEndTix) Then
            Slewing = True
        Else
            If Not m_bAbortSlew Then
                buf = m_Profile.GetValue(SCOPE_ID, "Beep")
                If buf = "1" Then _
                    Beep
            End If
            Slewing = False
            m_bAbortSlew = False
        End If
    End If
    m_bSlewing = Slewing                            ' Adjust our flag

End Property

Public Property Get SlewSettleTime() As Integer

    SlewSettleTime = m_iSettleTime
    
End Property

Public Property Let SlewSettleTime(ByVal newVal As Integer)

    If (newVal < 0) Or (newVal > 100) Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property SlewSettleTime " & MSG_PROP_RANGE_ERROR
        
    m_iSettleTime = newVal
    
End Property

' used for internal purposes only
Private Property Let TargetAltitude(ByVal newVal As Double)

    Dim sexa As String
    
'    CheckConnected False, m_bTracking, True
    
    If newVal < -90# Or newVal > 90# Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property TargetAltitude " & MSG_PROP_RANGE_ERROR
        
    If Not m_bLongFormat Then
        ' Short format, 1 decimal digit for LX200, no decimals for Autostar
        sexa = m_Util.DegreesToDM(newVal, m_sDecDelimDeg, m_sDecDelimMin, _
            IIf(m_eModel = modelAutostar Or m_eModel = modelAutostarGEM, 0, 1))
    Else
        sexa = m_Util.DegreesToDMS(newVal, m_sDecDelimDeg, _
            m_sDecDelimMin, m_sDecDelimSec) ' Long format, whole seconds
    End If
    If Left$(sexa, 1) <> "-" Then sexa = "+" & sexa         ' Both need leading '+'
    
    If Not Me.CommandBool("Sa " & sexa) Then
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
    End If
    
End Property

' used for internal purposes only
Private Property Let TargetAzimuth(ByVal newVal As Double)

    Dim sexa As String
    
'    CheckConnected False, m_bTracking, True
    
    If newVal < 0# Or newVal >= 360# Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property TargetAzimuth " & MSG_PROP_RANGE_ERROR
    
    If m_eModel <> modelGPS Then _
        newVal = RangeAz(newVal + 180#)
           
    If Not m_bLongFormat Then
        ' Short format, 1 decimal digit for LX200, no decimals for Autostar
        sexa = m_Util.DegreesToDM(newVal, m_sAzDelimDeg, m_sAzDelimMin, _
            IIf(m_eModel = modelAutostar Or m_eModel = modelAutostarGEM, 0, 1))
    Else
        sexa = m_Util.DegreesToDMS(newVal, m_sAzDelimDeg, _
            m_sAzDelimMin, m_sAzDelimSec) ' Long format, whole seconds
    End If
    
'    If Not m_bLongFormat Then
'        ' Short format, 1 decimal digit for LX200, no decimals for Autostar
'        sexa = m_Util.DegreesToDM(newVal, ":", ":", _
'            IIf(m_eModel = modelAutostar Or m_eModel = modelAutostarGEM , 0, 1))
'    Else
'        sexa = m_Util.DegreesToDMS(newVal, ":", _
'            ":", ":") ' Long format, whole seconds
'    End If
'    If Left$(sexa, 1) <> "-" Then sexa = "+" & sexa         ' Both need leading '+'
    
    If Not Me.CommandBool("Sz " & sexa) Then
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
    End If
    
    m_Profile.WriteValue SCOPE_ID, "BreadCrumbTA", "nv sexa " & newVal & " " & sexa
    
End Property

Public Property Get TargetDeclination() As Double

    CheckConnected False, m_bTracking, False
    
    If m_dTargetDec = INVALID_PARAMETER Then _
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
    
    TargetDeclination = m_dTargetDec
    
'    TargetDeclination = m_Util.DMSToDegrees(Me.CommandString("Gd"))

End Property

Public Property Let TargetDeclination(ByVal newVal As Double)

    Dim sexa As String
    
    CheckConnected False, m_bTracking, True
    
    If newVal < -90# Or newVal > 90# Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property TargetDeclination " & MSG_PROP_RANGE_ERROR
        
    If Not m_bLongFormat Then
        ' Short format, 1 decimal digit for LX200, no decimals for Autostar
        sexa = m_Util.DegreesToDM(newVal, m_sDecDelimDeg, m_sDecDelimMin, _
            IIf(m_eModel = modelAutostar Or m_eModel = modelAutostarGEM, 0, 1))
    Else
        sexa = m_Util.DegreesToDMS(newVal, m_sDecDelimDeg, _
            m_sDecDelimMin, m_sDecDelimSec) ' Long format, whole seconds
    End If
    If Left$(sexa, 1) <> "-" Then sexa = "+" & sexa         ' Both need leading '+'
    
    If Not Me.CommandBool("Sd " & sexa) Then
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
    End If
    
    m_dTargetDec = newVal
    
End Property

Public Property Get TargetRightAscension() As Double

    CheckConnected False, m_bTracking, False
    
    If m_dTargetRA = INVALID_PARAMETER Then _
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
        
    TargetRightAscension = m_dTargetRA
    
'    TargetRightAscension = m_Util.HMSToHours(Me.CommandString("Gr"))

End Property

Public Property Let TargetRightAscension(ByVal newVal As Double)
    Dim sexa As String
    
    CheckConnected False, m_bTracking, True
    
    If newVal < 0# Or newVal >= 24# Then _
        Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
            "Property TargetRightAscension " & MSG_PROP_RANGE_ERROR
    
    If Not m_bLongFormat Then
        sexa = m_Util.HoursToHM(newVal, m_sRADelimHrs, _
                            m_sRADelimMin, 1)              ' Short format, 1 decimal digit
    Else
        sexa = m_Util.HoursToHMS(newVal, m_sRADelimHrs, _
                            m_sRADelimMin, m_sRADelimSec)   ' Long format, whole seconds
    End If
    
    If Not Me.CommandBool("Sr " & sexa) Then
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
    End If
    
    m_dTargetRA = newVal
    
End Property

Public Property Get Tracking() As Boolean

    CheckConnected False, m_bTracking, False
    
    Tracking = (Not m_bParked) And m_bTracking  ' a robust test
    
End Property

Public Property Let Tracking(ByVal newVal As Boolean)

    CheckConnected newVal, m_bTracking, True
    
    ' autostar can't come back from tracking off, so don't support either
    
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then _
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
            "Control of Tracking" & MSG_NOT_IMPLEMENTED
    
    SetTracking newVal
    
End Property

Public Property Get TrackingRate() As DriveRates

    CheckConnected False, m_bTracking, False
    
    TrackingRate = m_eTrackingRate
    
End Property

Public Property Let TrackingRate(ByVal newVal As DriveRates)

    CheckConnected False, m_bTracking, False
    
    ' GPS should be more capable ???
    ' but the new commands fail under test... all code commented out for now
    
'    If m_eModel = modelGPS Then
'        ' only allow if scope is awake
'        If m_bAsleep Then _
'            Err.Raise SCODE_ASLEEP, ERR_SOURCE, MSG_ASLEEP
'        Select Case newVal
'            Case driveSidereal
'                Me.CommandBlind "TQ"
'            Case driveLunar
'                Me.CommandBlind "TL"
'            Case driveSolar
'                ' untested syntax ???
'                Me.CommandBlind "TM"
'                If Not Me.CommandBool("T060.164") Then _
'                    Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
'                        "Property TrackingRate " & MSG_PROP_RANGE_ERROR
'            Case Else
'                Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
'                    "Property TrackingRate " & MSG_PROP_RANGE_ERROR
'        End Select
'    Else
        If newVal <> driveSidereal Then _
            Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Property TrackingRate " & MSG_PROP_RANGE_ERROR
'    End If
        
    m_eTrackingRate = newVal
    
End Property

Public Property Get TrackingRates() As Object

    CheckConnected False, m_bTracking, False
    
    Set TrackingRates = m_TrackingRates
    
End Property

Public Property Get UTCDate() As Date

    CheckConnected False, m_bTracking, True
    
    UTCDate = CDate(Me.CommandString("GL"))                   ' Local time
    ' DO NOT USE CDate() here!!! Scope always returns MM/DD/YY even in Europe
    UTCDate = UTCDate + mdy_vb(Me.CommandString("GC"))        ' Calendar date
    UTCDate = UTCDate + (CDbl(Me.CommandString("GG")) / 24)   ' Now UTC

End Property

Public Property Let UTCDate(ByVal newVal As Date)
    Dim buf As String
    Dim tbuf As String
    Dim I As Integer
    Dim en As Boolean
    Dim UTCOffset As Integer
    
    CheckConnected False, m_bTracking, True
    
    '
    ' Autostar cannot recalculate its transform after alignment,
    ' thus changing the site location or time will invalidate its
    ' internal model. Disallow on Autostar.
    '
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Or m_eModel = modelGPS Then
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
                "Property Let UTCDate (Autostar)" & MSG_NOT_IMPLEMENTED
    End If
    
    '
    ' Set the scope's UTC offset to that of the PC, then convert
    ' UTC to local time and set that into the scope.
    '
    UTCOffset = CInt(utc_offs() / 3600)
    buf = Format$(UTCOffset, "00")
    If UTCOffset >= 0 Then buf = "+" & buf          ' Must have explicit +
    If Not Me.CommandBool("SG " & buf) Then
        Err.Raise SCODE_BAD_UTCOFFSET, ERR_SOURCE, MSG_BAD_UTCOFFSET
    End If
    
    '
    ' Next do the time, as date alerts planetary changes
    ' The LX200 recalculates ST when the LT changes. The
    ' Autostar does not.
    '
    newVal = CDate(CDbl(newVal) - (UTCOffset / 24#))    ' Convert to local time
    If Not Me.CommandBool("SL " & Format$(newVal, "hh:nn:ss")) Then
        Err.Raise SCODE_BAD_DATETIME, ERR_SOURCE, MSG_BAD_DATETIME
    End If

    '
    ' Make date string in scope format. Check the scope's
    ' date and if the same, skip the date set. Otherwise
    ' set it. We do things here so that various local date
    ' formats like 05.12.98 will work...
    '
    If mdy_vb(Me.CommandString("GC")) = CDate(Fix(newVal)) Then
        GoTo TS_OK                      ' That's it
    End If
    '
    ' OK, we're going to change the date...
    '
    g_SharedSerial.Transmit ":SC " & vb_mdy(Fix(newVal)) & "#"
    '
    ' Wait for the OK boolean then bail if the date was bad
    '
    buf = g_SharedSerial.ReceiveCounted(1)
    If Left$(buf, 1) = "0" Then     ' Illegal date
        Err.Raise SCODE_BAD_DATETIME, ERR_SOURCE, MSG_BAD_DATETIME
    End If
    '
    ' Wait for the "Updating..." then toss the blanks message
    '
    g_SharedSerial.ReceiveTerminated "#"
    g_SharedSerial.ReceiveTerminated "#"
    
TS_OK:

End Property

' -------
' Methods
' -------

Public Sub AbortSlew()

    CheckConnected True, m_bTracking, True
    
    Me.CommandBlind "Q"                             ' Halt the slew
    m_bAbortSlew = True                             ' Raise the abort flag
    m_Util.WaitForMilliseconds 1000                 ' Give the Autostar a second to react
    
End Sub

Public Function AxisRates(ByVal Axis As TelescopeAxes) As Object

    ' m_AxisRates is not set up till connect
    CheckConnected False, m_bTracking, False
    
    ' return empty collection for all Axis (that is proper for not supported)
    ' could support coarse rates for gps, but this wouldn't meet
    ' the real need at 0.1 deg/sec
    
    Set AxisRates = m_AxisRates

End Function

Public Function CanMoveAxis(ByVal Axis As TelescopeAxes) As Boolean
    
    ' not supported at this time, poor precision in the protocol (0.1 deg/sec)
    CanMoveAxis = False

End Function

Public Sub CommandBlind(ByVal Command As String, _
        Optional ByVal Raw As Boolean = False)

    g_SharedSerial.ClearBuffers                           ' Clear remaining junk in buffer
    
    If Raw Then
        g_SharedSerial.Transmit Command
    Else
        g_SharedSerial.Transmit ":" & Command & "#"
    End If
    
End Sub

'GC
Public Sub CommandBlind1(ByVal Command As String, _
        Optional ByVal Raw As Boolean = False)

    g_GuideSerial.ClearBuffers                           ' Clear remaining junk in buffer
    
    If Raw Then
        g_GuideSerial.Transmit Command
    Else
        g_GuideSerial.Transmit ":" & Command & "#"
    End If
    
End Sub
'/GC

Public Function CommandBool(ByVal Command As String, _
        Optional ByVal Raw As Boolean = False) As Boolean
    Dim buf As String
    
    g_SharedSerial.ClearBuffers                           ' Clear remaining junk in buffer
    
    If Raw Then
        g_SharedSerial.Transmit Command
    Else
        g_SharedSerial.Transmit ":" & Command & "#"
    End If
    
    buf = g_SharedSerial.ReceiveCounted(1)                ' Just a 1 or 0
    
    If buf = "1" Then
        CommandBool = True
    Else
        CommandBool = False
    End If

End Function

Public Function CommandString(ByVal Command As String, _
        Optional ByVal Raw As Boolean = False) As String
    Dim buf As String
    
    g_SharedSerial.ClearBuffers                           ' Clear remaining junk in buffer
    
    If Raw Then
        g_SharedSerial.Transmit Command
    Else
        g_SharedSerial.Transmit ":" & Command & "#"
    End If
    
    buf = g_SharedSerial.ReceiveTerminated("#")
    If buf <> "" Then                   ' Overflow protection
        If Raw Then
            CommandString = buf
        Else
            CommandString = Left$(buf, Len(buf) - 1)   ' Strip '#'
        End If
    Else
        CommandString = ""
    End If

End Function

Public Function DestinationSideOfPier(ByVal RightAscension As Double, _
                             ByVal Declination As Double) As PierSide

    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Method DestinationSideOfPier()" & MSG_NOT_IMPLEMENTED
    
    ' The Meade protocol does not have a pier flip command that we can find
               
End Function

Public Sub FindHome()

    ' some meade scopes could do this (gps) ???
    ' maybe kick off gps alignment (after meade firmware stabilizes) ???
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
                "Method FindHome()" & MSG_NOT_IMPLEMENTED

End Sub

Public Sub MoveAxis(ByVal Axis As TelescopeAxes, ByVal Rate As Double)
    
    ' not supported at this time, poor precision in the protocol (0.1 deg/sec)
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Method MoveAxis" & MSG_NOT_IMPLEMENTED

End Sub

Public Sub Park()

    Dim savedST As Integer
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim dec As Double
    Dim buf As String
    
    CheckConnected False, m_bTracking, True
    
    ' bail if already good
    If m_bParked Then _
        Exit Sub
        
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Or _
            m_eModel = modelGPS Or m_eModel = modelClassic16 Then
        m_bAsleep = True
        Me.CommandBlind "hP"
        If m_eModel = modelGPS Then _
            m_Profile.WriteValue SCOPE_ID, "GPSParked", "True"
    Else
        ' start by turning off tracking
        SetTracking False
  
        ' set up park position
        On Error Resume Next
        Alt = IIf(m_eAlignMode = algAltAz, 0#, 90# - Abs(m_dLat))                                 ' use default as seed
        buf = m_Profile.GetValue(SCOPE_ID, "ParkAlt")       ' fetch any saved val
        If buf <> "" Then _
            Alt = val(buf)                                  ' if saved, use it
        Az = IIf(m_dLat >= 0, 180#, 0#)                     ' use default as seed
        buf = m_Profile.GetValue(SCOPE_ID, "ParkAz")        ' fetch any saved val
        If buf <> "" Then _
            Az = val(buf)                                   ' if saved, use it
        On Error GoTo 0
        
        If m_eAlignMode = algAltAz Then
            TargetAltitude = Alt
            TargetAzimuth = Az
        Else
            ' convert to scope centric Alt/Az
            calc_hadec Az, Alt, m_dLat, ha, dec
            TargetAltitude = dec
            TargetAzimuth = RangeAz((ha * 15#) + 180#)
        End If
        
        ' override settle time (we're only parking)
        savedST = m_iSettleTime                     ' Save current settle time
        m_iSettleTime = 0                           ' Suspend settle time
        
        ' do the deed
        DoSlewAsync False

        ' parking is synchronous
        While Me.Slewing                            ' Wait for slew to complete
            m_Util.WaitForMilliseconds 200          ' Be responsive to AbortSlew()
        Wend
        
        ' Restore previous settle time
        m_iSettleTime = savedST
    End If
    
    ' update state
    m_bTracking = False
    m_bParked = True
    
End Sub

Public Sub PulseGuide(ByVal Direction As GuideDirections, ByVal Duration As Long)
    Dim d As String
    Dim ms As String
    
    If Duration < 0# Then _
        Err.Raise SCODE_PULSE_DUR, ERR_SOURCE, _
            "Method PulseGuide() " & MSG_PULSE_DUR
            
    CheckConnected True, m_bTracking, True
    
    Select Case Direction
        Case guideNorth:    d = "n"
        Case guideSouth:    d = "s"
        Case guideEast:     d = "e"
        Case guideWest:     d = "w"
    End Select
        
    If m_eModel = modelGPS And Duration <= 2000# Then
        ' use undocumented command deemed to be more robust, but its
        ' only valid to 2000 ms
                
        ms = Format$(Duration, "0000")
        Me.CommandBlind "Mg" & d & ms
        m_Util.WaitForMilliseconds Duration
    Else
'GC
    If m_eModel = modelAutostar And Duration <= 2000# Then
        ' use undocumented command deemed to be more robust, but its
        ' only valid to 2000 ms
        ms = Format$(Duration, "0000")
        If g_GuideSerial.Connected = True Then
            Me.CommandBlind1 "Mg" & d & ms
          Else
            Me.CommandBlind "Mg" & d & ms
        End If
        m_Util.WaitForMilliseconds Duration
'/GC
    Else
        Me.CommandBlind "RG"                ' Assure we're in Guide rate
        
        Me.CommandBlind "M" & d
        m_Util.WaitForMilliseconds Duration
        Me.CommandBlind "Q" & d
        
        ' classic only !!!, this is needed since once in a while one is not enough
        m_Util.WaitForMilliseconds 200
        Me.CommandBlind "Q" & d
    End If
    End If
    
End Sub

Public Sub SetPark()

    Dim Alt As Double
    Dim Az As Double
    Dim dec As Double
    Dim ha As Double

    CheckConnected False, m_bTracking, True
    
    If Not Me.CanSetPark Then _
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
            "Method SetPark()" & MSG_NOT_IMPLEMENTED

    Alt = Me.Altitude
    Az = Me.Azimuth
    
    ' make sure it can be distinguished from normal turn on, and give
    ' it a little margin also
    If m_eAlignMode = algAltAz Then
        If Az > 179.5 And Az < 180# + (10# * 15# / 60#) And _
                Alt > -0.5 And Alt < 0.5 Then _
            Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Method SetPark() " & MSG_PROP_RANGE_ERROR
    Else
        calc_hadec Az, Alt, m_dLat, ha, dec
        ' is south right ??? (don't really care yet, since south is not supported)
        If ha > -2# / 60# And ha < 10# / 60# And _
                dec > -0.5 And dec < 0.5 Then _
            Err.Raise SCODE_PROP_RANGE_ERROR, ERR_SOURCE, _
                "Method SetPark() " & MSG_PROP_RANGE_ERROR
    End If
    
    m_Profile.WriteValue SCOPE_ID, "ParkAlt", Str(Alt)
    m_Profile.WriteValue SCOPE_ID, "ParkAz", Str(Az)
    
End Sub

Public Sub SetupDialog()
    
    frmSetup.Show 1
    Unload frmSetup
    
End Sub

Public Sub SlewToAltAz(ByVal Azimuth As Double, _
                                    ByVal Altitude As Double)
    
    Me.SlewToAltAzAsync Azimuth, Altitude
    
    While Me.Slewing                            ' Wait for slew to complete
        m_Util.WaitForMilliseconds 200          ' Be responsive to AbortSlew()
    Wend
    
End Sub

Public Sub SlewToAltAzAsync(ByVal Azimuth As Double, ByVal Altitude As Double)
    
    Dim ST As Double
    Dim ha As Double
    Dim RA As Double
    Dim dec As Double
    Dim stat As Integer
    Dim buf As String
            
    CheckConnected True, False, True
            
    If m_bTracking Then

        ' alt/az slew won't work while tracking, convert to RA/Dec
        calc_hadec Azimuth, Altitude, m_dLat, ha, dec
        ST = SiderealTime
        RA = RangeRA(ST - ha)
        Me.TargetRightAscension = RA
        Me.TargetDeclination = dec
        DoSlewAsync True
    Else
    
        ' not tracking, so do a real alt/az slew
        If m_eAlignMode = algAltAz Then
            TargetAltitude = Altitude
            TargetAzimuth = Azimuth
        Else
            ' convert to scope centric Alt/Az
            calc_hadec Azimuth, Altitude, m_dLat, ha, dec
            TargetAltitude = dec
            TargetAzimuth = RangeAz((ha * 15#) + 180#)
        End If
        
        DoSlewAsync False
    End If
        
End Sub

Public Sub SlewToCoordinates(ByVal RightAscension As Double, _
                                    ByVal Declination As Double)
    
    SlewToCoordinatesAsync RightAscension, Declination
        
    While Me.Slewing                            ' Wait for slew to complete
        m_Util.WaitForMilliseconds 200          ' Be responsive to AbortSlew()
    Wend
    
End Sub

Public Sub SlewToCoordinatesAsync(ByVal RightAscension As Double, _
                                    ByVal Declination As Double)
    
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim ST As Double
    
    CheckConnected True, True, True
            
    Me.TargetRightAscension = RightAscension
    Me.TargetDeclination = Declination
    
    If m_bTracking Then
        DoSlewAsync True
    Else
        ST = SiderealTime
        ha = RangeHA(ST - RightAscension)
        If m_eAlignMode = algAltAz Then
            calc_altaz -ha, Declination, m_dLat, Az, Alt
            TargetAltitude = Alt
            TargetAzimuth = Az
        Else
            TargetAltitude = Declination
            TargetAzimuth = RangeAz((ha * 15#) + 180#)
        End If
        DoSlewAsync False
    End If
    
End Sub

Public Sub SlewToTarget()
    
    SlewToTargetAsync
    
    While Me.Slewing                            ' Wait for slew to complete
        m_Util.WaitForMilliseconds 200          ' Be responsive to AbortSlew()
    Wend
        
End Sub

Public Sub SlewToTargetAsync()
    
    Dim Alt As Double
    Dim Az As Double
    Dim ha As Double
    Dim RA As Double
    Dim dec As Double
    Dim ST As Double
    
    CheckConnected True, True, True
        
    If m_dTargetRA = INVALID_PARAMETER Or m_dTargetDec = INVALID_PARAMETER Then _
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
        
    If m_bTracking Then
        DoSlewAsync True                            ' Start slew
    Else
        ' tracking is off, get HA and Dec then convert to Alt/Az
        RA = Me.TargetRightAscension
        dec = Me.TargetDeclination
        ST = SiderealTime
        ha = RangeHA(ST - RightAscension)
        If m_eAlignMode = algAltAz Then
            calc_altaz -ha, Declination, m_dLat, Az, Alt
            TargetAltitude = Alt
            TargetAzimuth = Az
        Else
            ' in polar mode, alt/az is mount relative
            TargetAltitude = Declination
            TargetAzimuth = RangeAz((ha * 15#) + 180#)
        End If
        
        DoSlewAsync False
    End If

End Sub

Public Sub SyncToAltAz(ByVal Azimuth As Double, ByVal Altitude As Double)
    
    Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
        "Method SyncToAltAz()" & MSG_NOT_IMPLEMENTED
    
   ' unfortunatly, there is no altaz sync command code. We could convert
   ' to RA/DEC, but that fails when in LAND mode...
    
End Sub

Public Sub SyncToCoordinates(ByVal RightAscension As Double, _
                                    ByVal Declination As Double)
                                    
    CheckConnected True, True, True
        
    Me.TargetRightAscension = RightAscension
    Me.TargetDeclination = Declination
    
    If Me.CommandString("CM") = "" Then _
        Err.Raise SCODE_SYNC_FAILED, ERR_SOURCE, MSG_SYNC_FAILED
    
End Sub

Public Sub SyncToTarget()
                  
    CheckConnected True, True, True
        
    If m_dTargetRA = INVALID_PARAMETER Or m_dTargetDec = INVALID_PARAMETER Then _
        Err.Raise SCODE_BAD_TARGET, ERR_SOURCE, MSG_BAD_TARGET
        
    If Me.CommandString("CM") = "" Then _
        Err.Raise SCODE_SYNC_FAILED, ERR_SOURCE, MSG_SYNC_FAILED

End Sub

Public Sub Unpark()

    Dim buf As String
    Dim I As Integer
    
    CheckConnected False, m_bTracking, False
    
    ' No op if not parked
    If Not m_bParked Then _
        Exit Sub
        
    ' autostar can't come back from park state
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then _
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
            "Method Unpark()" & MSG_NOT_IMPLEMENTED
    
    ' what about the 16" ??? (more testing would be good)
    If m_eModel = modelGPS Then
        m_Profile.WriteValue SCOPE_ID, "GPSParked", "" ' This is a fragile state
        g_SharedSerial.ReceiveTimeout = 2           ' Switch to 2 sec timeout

        ' "I" is the only way out of hP driven Park
        Me.CommandBlind "I"                         ' LX200GPS "reboot"
        m_Util.WaitForMilliseconds 5000             ' Wait at least 5 sec
        On Error Resume Next
        For I = 1 To 11                             ' For up to 55 sec more
            buf = Me.CommandString("GS")            ' Try for sidereal time (2 sec timeout)
            If Err.Number = 0 Then Exit For         ' Got it!
            Err.Clear
            m_Util.WaitForMilliseconds 3000         ' Wait another 3 sec, for a total of 5 sec.
        Next I
        g_SharedSerial.ReceiveTimeout = 30          ' Switch back to 30 sec timeout
        On Error GoTo 0
        If I > 11 Then Err.Raise SCODE_GPSREBOOT_FAIL, ERR_SOURCE, _
                "Unpark() - " & MSG_GPSREBOOT_FAIL
        
        m_bParked = False
        m_bTracking = True
        m_bAsleep = False
    Else
          
        ' OK, we're not parked
        m_bParked = False
        ' Tada !
        
        ' see if tracking should be turned on, then do it
        buf = m_Profile.GetValue(SCOPE_ID, "AutoUnpark")
        If buf = "" Or buf = "1" Then _
            SetTracking True

    End If
    
End Sub

' ========================
' PRIVATE HELPER FUNCTIONS
' ========================

'
' CheckConnected() - Raise an error if the scope is not connected
'
Private Sub CheckConnected(Park As Boolean, Track As Boolean, Asleep As Boolean)

    If Not IsConnected(devTelescope) Then _
        Err.Raise SCODE_NOT_CONNECTED, ERR_SOURCE, MSG_NOT_CONNECTED
    
    ' only allow if scope needed to be awake
    If Asleep And m_bAsleep Then _
        Err.Raise SCODE_ASLEEP, ERR_SOURCE, MSG_ASLEEP
        
    ' Global error checking for all slews and syncs
    If Park And m_bParked Then _
        Err.Raise SCODE_PARKED, ERR_SOURCE, MSG_PARKED
    
    ' tracking checks for alt/az vs equ commands
    If Track <> m_bTracking Then _
        Err.Raise SCODE_WRONG_TRACKING, ERR_SOURCE, MSG_WRONG_TRACKING
        
End Sub

Private Sub DoSlewAsync(Polar As Boolean)
    Dim stat As Integer
    Dim msg As String
    Dim code As Long
    Dim buf As String
    Dim I As Integer
    Dim tmo As Integer
        
    '
    ' This is where all the work is done for slewing. The other
    ' methods eventually call this one. The LX200 GPS has a bug
    ' where once in a blue moon it will not respond to the MS
    ' command. Therefore, we retry it 5 times before giving up.
    '
    On Error GoTo CMD_VAPORIZED
    tmo = g_SharedSerial.ReceiveTimeout
    g_SharedSerial.ReceiveTimeout = 5
    For I = 0 To 6                                  ' Total of 30 sec...
        If Polar Then
            Me.CommandBlind "MS"
        Else
            Me.CommandBlind "MA"
        End If
        m_bAbortSlew = False
        m_bSlewing = True                           ' Could be a race here!
        stat = CInt(g_SharedSerial.ReceiveCounted(1))     ' Get only the start-status digit
        On Error GoTo 0                             ' Got a response...
        g_SharedSerial.ReceiveTimeout = tmo         ' Restore timeout
        If stat <> 0 Then   ' Failed to start slew
            msg = MSG_SLEW_FAIL
            code = SCODE_SLEW_FAIL
            If Polar Then
                Select Case stat
                    Case 1:                             ' Below horizon
                        msg = MSG_BELOW_HORIZ
                        code = SCODE_BELOW_HORIZ
                    Case 2:                             ' Below minimum elevation
                        msg = MSG_BELOW_MIN_EL
                        code = SCODE_BELOW_MIN_EL
                End Select
            End If
            m_bSlewing = False                      ' Clear slewing flag
            Err.Raise code, ERR_SOURCE, msg         ' FAILED
        End If
        GoTo CMD_ACCEPTED
CMD_VAPORIZED:
        Resume CMD_RETRY
CMD_RETRY:
    Next I
    On Error GoTo 0                                 ' Assure errors back on
    g_SharedSerial.ReceiveTimeout = tmo             ' Restore timeout
    Err.Raise SCODE_SLEW_FAIL, ERR_SOURCE, MSG_SLEW_FAIL    ' Failed!
    '
CMD_ACCEPTED:
    '
    ' OK, we're rollin' Save some state for the 16" classic mess
    '
    m_lSlewStartTix = GetTickCount()            ' Mark time of slew start
    m_lAsyncEndTix = GetTickCount() + (CLng(m_iSettleTime) * 1000&)
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then
        m_lAsyncEndTix = m_lAsyncEndTix + 11500 ' Start with a minimum of 11.5 sec
    ElseIf m_eModel = modelGPS Then
        m_lAsyncEndTix = m_lAsyncEndTix + 1000  ' Start with a minimum of 1 sec
    Else
        m_lAsyncEndTix = m_lAsyncEndTix + 4500  ' Start with a minimum of 4.5 sec
    End If
    If m_eModel = modelGPS Then
        m_Util.WaitForMilliseconds 2000         ' Get past GPS' dead time
    Else
        buf = Me.CommandString("D")             ' Save initial bar lengths (for 16" hack)
        m_iRaBar = Len(Trim$(Left$(buf, 16)))
        m_iDecBar = Len(Trim$(Right$(buf, 16)))
    End If
    
End Sub
'
' IsLongFormat() - Return True if scope is in long format
'
Private Function IsLongFormat() As Boolean
    Dim buf As String
    
    buf = Me.CommandString("GZ")            ' Get azimuth
    If Len(buf) <= 6 Then
        IsLongFormat = False                ' In short format
    Else
        IsLongFormat = True                 ' In long format
    End If

End Function
'
' SetLongFormat() - Set scope "long" format on or off
'
' The scope has only a toggle for the format, so we need to
' first test the format and then conditionally issue the
' toggle to set the desired format.
'
' Returns the format after the function is called. May
' fail on older LX200 scopes (V2.x ROMs).
'
Private Function SetLongFormat(b As Boolean) As Boolean
    Dim buf As String
    
    buf = Me.CommandString("GZ")            ' Get azimuth
    '
    ' Scope has some sort of race condition with "U", so
    ' we just wait a half sec. here and then do it.
    '
    m_Util.WaitForMilliseconds 500
    If ((b And (Len(buf) <= 6)) Or (Not b And (Len(buf) > 6))) Then
        Me.CommandBlind "U"                 ' change setting
    End If
    
    m_bLongFormat = IsLongFormat()          ' Return actual result
    SetLongFormat = m_bLongFormat
    
End Function

' centralized tracking control
Private Sub SetTracking(newVal As Boolean)
    
    ' only continue if we need to
    If newVal = m_bTracking Then _
        Exit Sub
    
    ' autostar can't change tracking state
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then _
        Err.Raise SCODE_NOT_IMPLEMENTED, ERR_SOURCE, _
            "Control of Tracking" & MSG_NOT_IMPLEMENTED
                
    If newVal Then
        ' tracking is not allowed direct from the park state
        If m_bParked Then _
            Err.Raise SCODE_PARKED, ERR_SOURCE, MSG_PARKED
    
        ' LX200 Classic: Changing from LAND to astronomical
        ' mode starts sidereal tracking.

        m_bInvalidCoords = True             ' handle race condition
        Select Case m_eAlignMode
            Case algAltAz:          Me.CommandBlind "AA"
            Case algPolar:          Me.CommandBlind "AP"
            Case algGermanPolar:    Me.CommandBlind "AP"
            Case Else:              Me.CommandBlind "AP"
        End Select
        
        m_bTracking = True                  ' OK, not tracking anymore
        m_Util.WaitForMilliseconds 1000     ' Mode takes time
        m_bInvalidCoords = False            ' coordinates valid again
        Me.TrackingRate = m_eTrackingRate
    Else

        ' turn off tracking by going to LAND Mode
        m_bInvalidCoords = True             ' handle race condition
        Me.CommandBlind "AL"                ' Switch to LAND mode
        m_bTracking = False                 ' OK, not tracking anymore
        m_Util.WaitForMilliseconds 1000     ' Land takes time
        m_bInvalidCoords = False            ' coordinates valid again
    End If
    
End Sub

' only called on connect and then only if the scope is found to be in
' LAND mode, thus potentially parked.  This routune determines if we
' really are parked, and what coords we should unpark to, then sets
' park state.
' It is assumed that Park state is False at this time
Private Sub Unsetpark()

    Dim Alt As Double, Az As Double     ' true scope position
    Dim ha As Double, RA As Double, dec As Double
    Dim pAlt As Double, pAz As Double   ' park position
    Dim ST As Double
    Dim buf As String
    Dim bAutoUnpark As Boolean          ' should we auto unpark?
    Dim bSetPark As Boolean             ' is park position set?
    Dim bColdStart As Boolean           ' is scope found in likely cold start?

    ' determine whether we will be automatically unparking
    buf = m_Profile.GetValue(SCOPE_ID, "AutoUnpark")
    bAutoUnpark = IIf(buf = "" Or buf = "0", False, True)
            
    ' most mounts can't handle the unsetpark routines ???
    If m_eModel = modelAutostar Or m_eModel = modelAutostarGEM Then
        m_bParked = True
        Exit Sub
    End If
    If m_eModel = modelGPS Then         ' Connect always unparks!
        m_bParked = False
    End If
    If m_eModel = modelClassic16 Then
        If bAutoUnpark Then
            Me.Unpark
        Else
            m_bParked = True
        End If
        Exit Sub
    End If
    
    ' fetch position to unpark to, default if necessary
    On Error Resume Next
    Alt = IIf(m_eAlignMode = algAltAz, 0#, 90# - Abs(m_dLat))   ' default
    pAlt = Alt                                                  ' use default as seed
    buf = m_Profile.GetValue(SCOPE_ID, "ParkAlt")               ' fetch any saved val
    If buf <> "" Then _
        pAlt = val(buf)                                         ' if saved, use it
        
    Az = IIf(m_dLat >= 0, 180#, 0#)                             ' default
    pAz = Az                                                    ' use default as seed
    buf = m_Profile.GetValue(SCOPE_ID, "ParkAz")                ' fetch any saved val
    If buf <> "" Then _
        pAz = val(buf)                                          ' if saved, use it
    On Error GoTo 0
    
    ' determine if park point is at a SetPark position
    bSetPark = IIf(pAlt = Alt And pAz = Az, False, True)
 
    ' figure out where we think we are are
    Az = m_Util.DMSToDegrees(Me.CommandString("GZ"))
    
    ' Wrap Az around 180 deg to the "real" azimuth.
    ' what does GPS wake up with (N and S) ??? (of course, its not supported yet anyway for unsetpark)
    If m_eModel <> modelGPS Then _
        Az = RangeAz(Az + 180#)
    
    Alt = m_Util.DMSToDegrees(Me.CommandString("GA"))
    
    ' detect power up state
    ' by checking mount centered Alt/Az for being within
    ' worm finding window (assume land mode)
    ' does south go the other way ??? (south not supported anyway)
    ' give Alt a range since it does not wake up at 0 exactly
    bColdStart = IIf(Az > 179.75 And Az < 180# + (9# * 15# / 60#) And _
            Alt > -0.25 And Alt < 0.25, True, False)
    
    m_Profile.WriteValue SCOPE_ID, "BreadCrumbUSP", _
        "setpark " & bSetPark & " cold " & bColdStart & _
        " Alt " & Alt & " Az " & Az
    
    ' calculate where we really are taking into account the worm
    ' convert to HA, Dec in preperation for low latency sync
    ' and or convert Alt/Az to horizon centric
    If bColdStart Then
        If m_eAlignMode = algAltAz Then
            Alt = RangeAlt(pAlt + Alt)
                
            Az = RangeAz(pAz + (Az - 180#))      ' subtract the boot up bias
            ' what is south lat start Az ??? 0 ? (south not supported anyway)
                    
            calc_hadec Az, Alt, m_dLat, ha, dec
        Else
            calc_hadec pAz, pAlt, m_dLat, ha, dec
            ha = RangeHA(ha + ((Az - 180#) / 15#))
            dec = RangeAlt(dec + Alt)
            calc_altaz -ha, dec, m_dLat, Az, Alt
        End If
    Else
        If m_eAlignMode = algAltAz Then
            calc_hadec Az, Alt, m_dLat, ha, dec
        Else
            ha = RangeHA((Az - 180#) / 15#)
            dec = Alt
            calc_altaz -ha, dec, m_dLat, Az, Alt
        End If
    End If

    ' This also fixes the old southern hemisphere bug where
    ' unpark after power cycle didn't fix up the coords
    If bSetPark Or m_dLat < 0 Then
        If bColdStart Then
                
            ' get sidereal now so that is matches HA at start of SetTracking
            ST = SiderealTime
            
            ' start tracking since we can only sync when tracking
            SetTracking True
            
            ' calculate RA
            RA = RangeRA(ST - ha)
                        
            ' send our real coordinates
            SyncToCoordinates RA, dec
        End If
    End If
    
    ' Get the final state right
    If bAutoUnpark Then
        ' if there was no sync, we won't be tracking yet
        If Not m_bTracking Then _
            SetTracking True
    Else
        ' we may have left tracking on in order to sync
        If m_bTracking Then _
            SetTracking False
        m_bParked = bColdStart
        
        ' if scope was at parked position but not power cycled then
        ' true park state is yet to be detected
        If Not m_bParked Then
            If Abs(Alt - pAlt) < 0.1 And _
                    (Abs(Az - pAz) < 0.1 Or Abs(Az - pAz) > 359.9) Then _
                m_bParked = True
        End If
    End If
    
End Sub

'' checks that a guiding rate commanded is in range
'Private Function RateIsInRange(Rate As Double) As Boolean
''    Dim r As range
''
''    For Each r In m_GuideRanges
''        If Abs(Rate) <= r.Maximum And Abs(Rate) >= r.Minimum Then
''            RateIsInRange = True
''            Exit Function
''        End If
''    Next
'    RateIsInRange = False
'End Function

'
' Special formatter for Lat/Long
'
Private Function DegtoDM(ByVal d As Double, ByVal p As Integer) As String
    Dim h As String
    Dim m As String
    Dim df As String
    Dim n As Boolean
    
    If d < 0# Then
        d = -d
        n = True
    Else
        n = False
    End If
    
    If p = 2 Then df = "00" Else df = "000"
    
    h = Format$(Fix(d), df)             ' Whole degrees
    d = (d - CDbl(h)) * 60#             ' Minutes
    m = Format$(CInt(d), "00")          ' Whole minutes (rounded)
    If m = "60" Then
        m = "00"
        h = Format$((CInt(h) + 1), df)
    End If

    DegtoDM = h & Chr$(223) & m
    If n Then DegtoDM = "-" & DegtoDM
       
End Function

'
' Convert VB style date/time to MM/DD/YY for use as input to the scope
'
Private Function vb_mdy(dt As Date) As String
    Dim yy As Integer
    
    dt = CVDate(Fix(dt))
    yy = Year(dt) Mod 100
    vb_mdy = Format$(Month(dt), "00") & "/" & _
            Format$(Day(dt), "00") & "/" & _
            Format$(yy, "00")

End Function

Private Function RangeAlt(ByVal Alt As Double)

    If Alt > 90# Then _
        Alt = 90#
    If Alt < -90# Then _
        Alt = -90#
             
    RangeAlt = Alt
    
End Function

Private Function RangeAz(ByVal Az As Double)

    While Az >= 360#
        Az = Az - 360#
    Wend
    While Az < 0#
        Az = Az + 360#
    Wend
             
    RangeAz = Az
    
End Function

Private Function RangeHA(ByVal ha As Double)
    
    While ha > 12#
        ha = ha - 24#
    Wend
    While ha <= -12#
        ha = ha + 24#
    Wend
             
    RangeHA = ha
    
End Function

Private Function RangeRA(ByVal RA As Double)

    While RA >= 24#
        RA = RA - 24#
    Wend
    While RA < 0#
        RA = RA + 24#
    Wend
             
    RangeRA = RA
    
End Function

' ---------
' EquDist() - Return equatorial distance between objects, degrees
' ---------

Private Function EquDist(ra1, dec1, ra2, dec2)

    Dim x1, x2, y1, y2, z1, z2, PI, k, r
    Dim ca1, sa1, cd1, sd1, ca2, sa2, cd2, sd2
    Dim a1rad, d1rad, a2rad, d2rad

    PI = 4# * Atn(1#)
    k = PI / 180#

    a1rad = 15# * k * ra1
    a2rad = 15# * k * ra2
    d1rad = k * dec1
    d2rad = k * dec2
      
    ca1 = Cos(a1rad)
    ca2 = Cos(a2rad)
    sa1 = Sin(a1rad)
    sa2 = Sin(a2rad)

    cd1 = Cos(d1rad)
    cd2 = Cos(d2rad)
    sd1 = Sin(d1rad)
    sd2 = Sin(d2rad)

    x1 = cd1 * ca1
    x2 = cd2 * ca2
    y1 = cd1 * sa1
    y2 = cd2 * sa2
    z1 = sd1
    z2 = sd2

    r = x1 * x2 + y1 * y2 + z1 * z2

    If r > 1# Then r = 1#
    If r < -1# Then r = -1#

    EquDist = (1# / k) * Arccos(r)
        
End Function

Private Function Arccos(X)

    If X = 1# Then
      Arccos = 0#
    ElseIf X = -1# Then
      Arccos = 4# * Atn(1#)
    Else
      Arccos = 2# * Atn(1#) - Atn(X / Sqr(1# - X * X))
    End If

End Function

'---------------------------------------------------------------------
'
' calc_altaz() - Calculate current azimuth and altitude
'
'---------------------------------------------------------------------

Private Sub calc_altaz(ByVal ha As Double, ByVal dec As Double, _
        lat As Double, Az As Double, Alt As Double)
        
    Dim tAz As Double
    Dim tAlt As Double
    
    hadec_aa lat * DEG_RAD, ha * HRS_RAD, dec * DEG_RAD, tAlt, tAz
             
    Alt = tAlt * RAD_DEG
    
    Az = 360# - (tAz * RAD_DEG)
    
End Sub

'---------------------------------------------------------------------
'
' calc_hadec() - Calculate current hour angle  and declination
'
'---------------------------------------------------------------------

Private Sub calc_hadec(ByVal Az As Double, ByVal Alt As Double, _
        lat As Double, ha As Double, dec As Double)
        
    Dim tHA As Double
    Dim tRA As Double
    Dim tDec As Double
    
    aa_hadec lat * DEG_RAD, Alt * DEG_RAD, (360# - Az) * DEG_RAD, tHA, tDec
    
    ha = -(tHA * RAD_HRS)
    
    dec = tDec * RAD_DEG
    
End Sub
